/* Includes */ 
#include <linux/kernel.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/wait.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/poll.h>
#include <linux/device.h>
#include <linux/interrupt.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/irq.h>
#include <asm/hardware.h>
static int bat_disabled;

#define BATTERY_AVERAGE_SIZE 50
#define BATTERY_DEVNAME				"battery"
#define BATTERY_MAJOR				241

typedef struct {
	unsigned short u16BatteryVoltage;		/* battery voltage */
	unsigned short u16ChargeCurrent;		/* charge current */
} BATTERY_STATUS;

#define BATTERY_DRIVER_MAGIC		'B' /* Battery driver magic number */
#define IOR_BATTERY_STATUS			_IOR(BATTERY_DRIVER_MAGIC, 3, BATTERY_STATUS)
// not implemented, and not used:
#define IOW_BATTERY_CHARGER_ENABLE	_IO(BATTERY_DRIVER_MAGIC, 4)
#define IOW_BATTERY_CHARGER_DISABLE	_IO(BATTERY_DRIVER_MAGIC, 5)


static unsigned short batteryVoltageBuffer[BATTERY_AVERAGE_SIZE];
static unsigned short batteryCurrentBuffer[BATTERY_AVERAGE_SIZE];

static int batteryWait = BATTERY_AVERAGE_SIZE;
static int batteryVoltageSum = 0;
static int batteryCurrentSum = 0;

static int bat_hw_init(void) 
{
	int i;
	for (i=0; i<BATTERY_AVERAGE_SIZE; i++) {
		batteryCurrentBuffer[i] = 0;
		batteryVoltageBuffer[i] = 0;
	}
	batteryVoltageSum = 0;
	batteryCurrentSum = 0;
	batteryWait = BATTERY_AVERAGE_SIZE;
	return 0;
}

#if 0
static void bat_adc_poll(short adcBuffer[ADC_CHANNELS])
{
	// voltage = (1.2 * strBatteryStatus.u16BatteryVoltage * 2 / strBatteryStatus.u16ReferenceVoltage);
	// ampere = (1200.0 * strBatteryStatus.u16ChargeCurrent / (strBatteryStatus.u16ReferenceVoltage * 1500));

	int sample;
	sample = (adcBuffer[ADC_BAT]  * 2450) / adcBuffer[ADC_REF];
	batteryVoltageSum -= batteryVoltageBuffer[batteryIndex];
	batteryVoltageSum += sample;
	batteryVoltageBuffer[batteryIndex] = sample;

	sample = (adcBuffer[ADC_CHRG] * 942) / adcBuffer[ADC_REF];
	batteryCurrentSum -= batteryCurrentBuffer[batteryIndex];
	batteryCurrentSum += sample;
	batteryCurrentBuffer[batteryIndex] = sample;

	batteryIndex++;
	if (batteryIndex >= BATTERY_AVERAGE_SIZE) batteryIndex = 0;
	if (batteryWait > 0) batteryWait--;
}
#endif

static int bat_get_voltage(void)
{
	return batteryVoltageSum / BATTERY_AVERAGE_SIZE;
}

static int bat_get_current(void)
{
	return batteryCurrentSum / BATTERY_AVERAGE_SIZE;
}

static int bat_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	int ret;

	if (bat_disabled)
		return -ENODEV;

	switch (cmd) {
	case IOR_BATTERY_STATUS:
		if (!access_ok(VERIFY_WRITE, (void __user *) arg, sizeof(BATTERY_STATUS))) {
			ret = -EFAULT;
		} else {
			BATTERY_STATUS bs;
			bs.u16BatteryVoltage   = bat_get_voltage();
			bs.u16ChargeCurrent    = bat_get_current();
			ret = __copy_to_user((void __user *) arg, &bs, sizeof(BATTERY_STATUS)) ? -EFAULT : 0;
		}
		break;
	default:

		ret = -EINVAL;
		break;
	}

	return ret;
}

static int bat_open(struct inode *inode, struct file *file)
{
	return nonseekable_open(inode, file);
}

static int bat_release(struct inode *inode, struct file *file)
{
	return 0;
}

/* Kernel interface */
static struct file_operations bat_fops = {
	.owner		= THIS_MODULE,
	.ioctl		= bat_ioctl,
	.open		= bat_open,
	.release	= bat_release,
};

static int bat_probe(void)
{
	int ret;

	ret = bat_hw_init();
	if (ret < 0) {

		return ret;
	}

#if 0
	ret = adc_register_poll(bat_adc_poll);
	if (ret < 0) {

		return ret;
	}
#endif

	ret = register_chrdev(BATTERY_MAJOR, BATTERY_DEVNAME, &bat_fops);
	if (ret != 0) {

		return ret;
	}


	return 0;
}

static int bat_remove(void)
{

	unregister_chrdev(BATTERY_MAJOR, BATTERY_DEVNAME);


	return 0;
}

static int __init bat_mod_init(void)
{
  return bat_probe();
}

static void __exit bat_mod_exit(void)
{
  bat_remove();
}

module_init(bat_mod_init);
module_exit(bat_mod_exit);

MODULE_AUTHOR("Dimitry Andric <dimitry.andric@tomtom.com>");
MODULE_DESCRIPTION("TomTom GO Battery Driver");
MODULE_LICENSE("GPL");

/* EOF */
