#include <linux/kernel.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/input.h>
#include <linux/poll.h>



#define GPIO_DEVNAME					"hwstatus"
#define GPIO_MAJOR						240

typedef struct {
	unsigned char u8ChargeStatus;
	unsigned short u8InputStatus;
	unsigned char u8DockStatus;
} HARDWARE_STATUS;

static HARDWARE_STATUS gpio_hw_status;

#define GPIO_DRIVER_MAGIC				'U'
#define IOR_HWSTATUS					_IOR(GPIO_DRIVER_MAGIC, 0, HARDWARE_STATUS)

static int gpio_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{

  switch (cmd) {
	case IOR_HWSTATUS:
		return copy_to_user((void __user *) arg, &gpio_hw_status, sizeof gpio_hw_status) ? -EFAULT : 0;
		break;
 }

        return 0;
}

static int gpio_open(struct inode *inode, struct file *file)
{
	gpio_hw_status.u8ChargeStatus = 2;
	gpio_hw_status.u8InputStatus = 0;
	gpio_hw_status.u8DockStatus = 3;
	return nonseekable_open(inode, file);
}

static int gpio_release(struct inode *inode, struct file *file)
{
	return 0;
}

static struct file_operations gpio_fops = {
	.owner		= THIS_MODULE,
	.ioctl		= gpio_ioctl,
	.open		= gpio_open,
	.release	= gpio_release,
};

static int __init gpio_mod_init(void)
{
	register_chrdev(GPIO_MAJOR, GPIO_DEVNAME, &gpio_fops);
	return 0;
}

static void __exit gpio_mod_exit(void)
{
	unregister_chrdev(GPIO_MAJOR, GPIO_DEVNAME);
}

module_init(gpio_mod_init);
module_exit(gpio_mod_exit);

MODULE_AUTHOR("Rasmus Rohde");
MODULE_DESCRIPTION("TomTom GPIO Driver");
MODULE_LICENSE("GPL");
