#include <linux/kernel.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/input.h>
#include <linux/poll.h>

#define TS_DEVNAME					"ts"
#define TS_MAJOR					254

static wait_queue_head_t key_wait;

typedef struct {
        short pressure;
        short x;
        short y;
        short pad;
} TS_EVENT;

static ssize_t ts_read(struct file *file, char __user *data, size_t len, loff_t *ppos)
{
	TS_EVENT ev;

	if (file->f_flags & O_NONBLOCK) {
		return -EAGAIN;
	} else {
		DECLARE_WAITQUEUE(wait, current);
		
		add_wait_queue(&key_wait, &wait);
		set_current_state(TASK_INTERRUPTIBLE);
		for (;;) {
			if (signal_pending(current)) {
				set_current_state(TASK_RUNNING);
				remove_wait_queue(&key_wait, &wait);
				return -ERESTARTSYS;
			}
			schedule();
		}
		set_current_state(TASK_RUNNING);
		remove_wait_queue(&key_wait, &wait);
	}

	ev.x = 30;
	ev.y = 30;
	ev.pressure = 0xffff;
	ev.pad = 0;

	return __copy_to_user(data, &ev, sizeof(TS_EVENT)) ? -EFAULT : sizeof(TS_EVENT);
}


static unsigned int ts_poll(struct file *file, struct poll_table_struct *wait)
{
	poll_wait(file, &key_wait, wait);

	return 0;
}



static int ts_open(struct inode *inode, struct file *file)
{
	return nonseekable_open(inode, file);
}

static int ts_release(struct inode *inode, struct file *file)
{
	return 0;
}

static int ts_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
  return 0;
}

static struct file_operations ts_fops = {
	.owner		= THIS_MODULE,
	.read		= ts_read,
	.poll		= ts_poll,
	.ioctl		= ts_ioctl,
	.open		= ts_open,
	.release	= ts_release,
};

static int __init ts_mod_init(void)
{
	register_chrdev(TS_MAJOR, TS_DEVNAME, &ts_fops);
	return 0;
}

static void __exit ts_mod_exit(void)
{
	unregister_chrdev(TS_MAJOR, TS_DEVNAME);
}

module_init(ts_mod_init);
module_exit(ts_mod_exit);

MODULE_AUTHOR("Rasmus Rohde");
MODULE_DESCRIPTION("TomTom TS Driver");
MODULE_LICENSE("GPL");
